#ifndef MODULES_LIDAR_INNO_FRONT_INCLUDE_DATAWRAPPER_H_
#define MODULES_LIDAR_INNO_FRONT_INCLUDE_DATAWRAPPER_H_

#include <time.h>
#include <sys_ctr.h>
#include <queue>

#include "pub/PubDataWrapper.h"
#include "sdk_common/inno_lidar_packet.h"

template<typename EventType>
class DataWrapper: public PubDataWrapper<EventType>
{
private:
	std::string frame_name = "iv_points";
	std::vector<InnoDataPacket*> _frameData;
	std::vector<uint8_t> _data;
	uint64_t _pointsCount;

	std::queue<InnoDataPacket*> _processQueue;
	std::mutex _mutex;

	int _framesCount;
	int _frameIndex;
	bool _packetComplete;

	void pushFloatToUint8Vector(float value,
			ara::core::Vector<uint8_t> &uint8Vector)
	{
		// Convert the float value to an array of bytes using a byte representation
		uint8_t bytes[sizeof(float)];
		memcpy(bytes, &value, sizeof(float));

		// Push each byte individually into the uint8_t vector
		for (int i = 0; i < sizeof(float); i++)
		{
			uint8Vector.push_back(bytes[i]);
		}
	}

	void pushSizeToUint8Vector(uint16_t value, uint8_t size,
			ara::core::Vector<uint8_t> &uint8Vector)
	{

		uint8_t bytes[2];
		memcpy(bytes, &value, size);

		for (uint8_t i = 0; i < size; i++)
		{
			uint8Vector.push_back(bytes[i]);
		}
	}

public:
	void GetData(ara::com::SampleAllocateePtr<EventType> &data) override
	{
		//data->pointStep = 30;
		data->header.sensor_name = 0;
		timespec ts;
		clock_gettime(CLOCK_VIRTUAL, &ts);
		data->header.stampHigh = ts.tv_sec;
		data->header.stampLow = ts.tv_nsec;
	}

	void NewData(InnoDataPacket *frameData)
	{
		if (frameData != nullptr)
		{
			std::lock_guard<std::mutex> lock(_mutex);
			_processQueue.push(frameData);
		}
	}

	void Process(ara::com::SampleAllocateePtr<EventType> &dataPtr)
	{
		if (_packetComplete || _processQueue.empty())
			return;

		InnoDataPacket *frameData;
		{
			std::lock_guard<std::mutex> lock(_mutex);
			frameData = _processQueue.front();

			if (_frameIndex == -1)
			{
				_frameIndex = frameData->idx;

			}

			if (_frameIndex != frameData->idx)
			{
				_packetComplete = true;
				return;
			}

			_processQueue.pop();
		}

		GetData(dataPtr);

		for (uint32_t i = 0; i < frameData->item_number; i++)
		{
			perception::PointXYZIT p;
			p.x = frameData->en_xyz_points[i].x;
			p.y = frameData->en_xyz_points[i].y;
			p.z = frameData->en_xyz_points[i].z;
			p.intensity = frameData->en_xyz_points[i].intensity;
			dataPtr->points.push_back(p);
		}

		free(frameData);
		++_framesCount;
	}

	bool isPacketComplete()
	{
		return _packetComplete;
	}

	bool needProcess()
	{
		return !_processQueue.empty();
	}

	void Reset()
	{
		_packetComplete = false;
		_frameIndex = -1;
		_framesCount = 0;
		_pointsCount = 0;
	}

	void SetPoints(std::vector<InnoDataPacket*> frameData)
	{
		_frameData = frameData;
	}
};

#endif /* MODULES_LIDAR_INNO_FRONT_INCLUDE_DATAWRAPPER_H_ */
